//
//  NSObject+CommonObjects.m
//  LLJogTest
//
//  Created by Lawrence Lomax on 27/02/2010.
//  Copyright 2010 Lawrence Lomax. All rights reserved.
//

#import "NSObject+LLCommonData.h"
#import "LLCoreDataManager.h"
#import "LLUserDefaults.h"

NSString * const twitterRequestsForIdentifier = @"twitterRequestsForIdentifier";
NSString * const twitterUsersForUsername = @"twitterUsersForUsername";

@implementation NSObject (LLCommonData)

#pragma mark Twitter Requests
-(LLTwitterRequest *)twitterRequestWithIdentifier:(NSString *)requestIdentifier{
	NSDictionary * substitutions = [NSDictionary dictionaryWithObject:requestIdentifier forKey:@"request_identifier"];
	NSFetchRequest * fetchRequest = [[LLCoreDataManager sharedLLCoreDataManager] fetchRequestFromModelWithName:twitterRequestsForIdentifier withSubstitutions:substitutions];
	return (LLTwitterRequest *)[[LLCoreDataManager sharedLLCoreDataManager] firstObjectWithFetchRequest:fetchRequest];
}

#pragma mark Twitter Users
-(LLTwitterUser *)twitterUserForUsername:(NSString *)username createIfAbsent:(BOOL)create{
	NSDictionary * substitutions = [NSDictionary dictionaryWithObject:username forKey:@"username"];
	NSFetchRequest * fetchRequest = [[LLCoreDataManager sharedLLCoreDataManager] fetchRequestFromModelWithName:twitterUsersForUsername withSubstitutions:substitutions];
	LLTwitterUser * twitterUser = (LLTwitterUser *)[[LLCoreDataManager sharedLLCoreDataManager] firstObjectWithFetchRequest:fetchRequest];
	if(!twitterUser && create){
		twitterUser = (LLTwitterUser *)[[LLCoreDataManager sharedLLCoreDataManager] newObjectForName:@"LLTwitterUser"];
		[twitterUser setUsername:username];
	}
	return twitterUser;
}

-(NSArray *)followingTwitterUsersForCurrentUser{
	LLTwitterUser * currentTwitterUser = [[LLUserDefaults sharedLLUserDefaults] currentTwitterUser];
	return [currentTwitterUser followingUsers];
}

#pragma mark Twitter Statuses
-(NSArray *)managedTwitterStatusesFromDictionary:(NSArray *)tweets{
    NSMutableArray * array = [NSMutableArray arrayWithCapacity:[tweets count]];
    for(NSDictionary * dict in tweets){
        LLTwitterStatus * status = (LLTwitterStatus *)[[LLCoreDataManager sharedLLCoreDataManager] newObjectForName:@"LLTwitterStatus"];
        NSDictionary * userdict = [dict objectForKey:@"user"];
		LLTwitterUser * user = [self twitterUserForUsername:[userdict objectForKey:@"screen_name"] createIfAbsent:YES];
        [status setUser:user];
        [status setMessage:[dict objectForKey:@"text"]];
        [status setDate:[dict objectForKey:@"createdAt"]];		
        [array addObject:status];
    }
    NSLog(@"%@",array);
    return array;
}

-(NSArray *)managedTwitterUsersFromDictionaries:(NSArray *)twitterUsers{
    NSMutableArray * array = [NSMutableArray arrayWithCapacity:[twitterUsers count]];
    for(NSDictionary * dict in twitterUsers){
        LLTwitterUser * user = (LLTwitterUser *)[[LLCoreDataManager sharedLLCoreDataManager] newObjectForName:@"LLTwitterUser"];
        [user setUsername:[dict objectForKey:@"screen_name"]];
        [array addObject:user];
    }
    NSLog(@"%@",array);
    return array;
}

#pragma mark Journeys
-(LLJourney *)newJourneyStartingActive:(BOOL)startingActive{
	LLJourney * journey = (LLJourney *)[[LLCoreDataManager sharedLLCoreDataManager] newObjectForName:@"LLJourney"];
	if(startingActive){
		[[LLUserDefaults sharedLLUserDefaults] setActiveJourney:journey];
	}
	return journey;
}

#pragma mark GeoPoint

-(LLLocation *)lastBroadcastLocationForJourney:(LLJourney *)journey{
	NSDictionary * substitutions = [NSDictionary dictionaryWithObject:@"JOURNEY" forKey:journey];
	NSFetchRequest * fetchRequest = [[[LLCoreDataManager sharedLLCoreDataManager] managedObjectModel] fetchRequestFromTemplateWithName:@"broadcastLocationsForJourney" substitutionVariables:substitutions];
	NSSortDescriptor * sortDescrpitor = [[NSSortDescriptor alloc] initWithKey:@"timestamp" ascending:YES];
	[fetchRequest setSortDescriptors:[NSArray arrayWithObject:sortDescrpitor]];
	NSError * error;
	NSArray * locationsBroadcast = [[[LLCoreDataManager sharedLLCoreDataManager] managedObjectContext] executeFetchRequest:fetchRequest error:&error];
	if(locationsBroadcast){
		if([locationsBroadcast count] > 0){
			return [locationsBroadcast objectAtIndex:0];
		}else{
			return nil;
		}
	}else{
		return nil;
	}
}


@end
