//
//  LLJourneyController.m
//  LLJogTest
//
//  Created by Lawrence Lomax on 12/01/2010.
//  Copyright 2010 Lawrence Lomax. All rights reserved.
//

#import "LLJourneyController.h"
#import "LLHelperMacros.h"
#import "NSObject+AllCoreData.h"
#import "LLUserDefaults.h"

#define DEFAULT_MINIMUM_TIME_FOR_UPDATE 20  //Minimum distance before notifying delegate of location update
#define DEFAULT_MINIMUM_DISTANCE_FOR_UPDATE 60  //Minimum distance before broadcasting the location
#define MINIMUM_REQUIRED_ACCURACY 50    //Minimum accuracy in meters

@implementation LLJourneyController

@synthesize delegate, activeJourney;

#pragma mark Singleton Methods

SYNTHESIZE_SINGLETON_FOR_CLASS(LLJourneyController);

#pragma mark init

-(id)init{
    if(self = [super init]){
		clManager = [[CLLocationManager alloc] init];
        clManager.delegate = self;
        clManager.desiredAccuracy = kCLLocationAccuracyBest;
        clManager.distanceFilter = DEFAULT_MINIMUM_DISTANCE_FOR_UPDATE/4;
        
        if(clManager.headingAvailable)
            clManager.headingFilter = 10;
		
        activeJourney = [[LLUserDefaults sharedLLUserDefaults] activeJourney];
    }
    return self;
}

-(void)resumeJourney{
	if(![activeJourney startDate]){	//Journey Hasnt Started
		[activeJourney setStartDate:[NSDate date]];
	}
    [clManager startUpdatingLocation];
}

-(void)pauseJourney{
    [clManager stopUpdatingLocation];
}

-(void)endJourney{
    [clManager stopUpdatingLocation];
    [activeJourney setEndDate:[NSDate date]];
}

-(void)locationUpdate:(LLLocation *)newLocation withSufficientAccuracy:(BOOL)accurate{
	
}

-(void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation{
    LLLocation * lastLocationUpdate = [self lastBroadcastLocationForJourney:activeJourney];
	
	LLLocation * location = (LLLocation *)[self newObjectForName:@"LLLocation"];
	[location setTimestamp:newLocation.timestamp];
	[location setSpeedValue:newLocation.speed];
	[location setCourseValue:newLocation.course];

	LLGeoPointWithBounds * position = (LLGeoPointWithBounds *)[NSEntityDescription insertNewObjectForEntityForName:@"LLGeoPointWithBounds" inManagedObjectContext:[self managedObjectContext]];
	[position setLatitudeValue:newLocation.coordinate.latitude];
	[position setLongitudeValue:newLocation.coordinate.longitude];

	[location setGeoPoint:position];

	/* In order to send an update the accuracy must be within the required range */
	if(newLocation.horizontalAccuracy < MINIMUM_REQUIRED_ACCURACY && newLocation.verticalAccuracy < MINIMUM_REQUIRED_ACCURACY){
		if(lastLocationUpdate){ //The last location must be within range to know whether it has moved required distance
			/* The time since the last update must be above the threshold */
			NSTimeInterval timeSinceUpdateTriggered = [[lastLocationUpdate timestamp] timeIntervalSinceDate:newLocation.timestamp];
			if(timeSinceUpdateTriggered > DEFAULT_MINIMUM_TIME_FOR_UPDATE){
				[location setBroadcastValue:YES];
			}else{
				[location setBroadcastValue:NO];
			}
		}else{
			[location setBroadcastValue:YES];
		}
	}else{
		[location setBroadcastValue:NO];
	}
	
	if([location broadcastValue]){
		[delegate locationUpdate:location withSufficientAccuracy:YES];
	}else{
		[delegate locationUpdate:location withSufficientAccuracy:NO];
	}
	
	[activeJourney addLocationsObject:location];

}

-(void)startUpdates{
    [clManager startUpdatingLocation];
    
    if(clManager.headingAvailable)    
        [clManager startUpdatingHeading];
}

-(void)stopUpdates{
    [clManager stopUpdatingLocation];
    
    if(clManager.headingAvailable)
        [clManager stopUpdatingLocation];
}

- (void)locationManager:(CLLocationManager *)manager didUpdateHeading:(CLHeading *)newHeading{
   NSLog(@"Heading x=%f , y=%f, z=%f",newHeading.x,newHeading.y,newHeading.z);
   
}

-(void)dealloc{
    [activeJourney release];    
    [super dealloc];
}

@end
