//
//  LLTwitterController.m
//  MKTest
//
//  Created by Lawrence Lomax on 23/12/2009.
//  Copyright 2009 Lawrence Lomax. All rights reserved.
//

#import "MGTwitterEngine.h"
#import "LLTwitterController.h"
#import "LLNotificationStrings.h"
#import "LLManagedObjects.h"
#import "NSObject+AllCoreData.h"
#import "LLUserDefaults.h"

#define MAX_MESSAGE_LENGTH 140

NSString * const JogcastrHashtag = @"#jogcastr";
NSString * const NotifyUsersStartRace = @"startrace";
NSString * const NotifyUsersEndRace = @"endrace";

@implementation LLTwitterController

@synthesize delegate;

#pragma mark Singleton Implementation

SYNTHESIZE_SINGLETON_FOR_CLASS(LLTwitterController)

#pragma mark Controller Implementation

-(id)init{
    if(self == [super init]){        
        twitterEngine = [[MGTwitterEngine alloc] initWithDelegate:self];        
			
        [self setCredentialsFromStore];
        
        //[self getStoreCredentials];
        //[self checkCurrentCredentials];
    }
    return self;
}

-(void)dealloc{
    [twitterEngine release];
    
    [super dealloc];
}

#pragma mark Implementation of MGTwitterEngineDelegate
- (void)requestSucceeded:(NSString *)requestIdentifier{
    LLTwitterRequest * req = [self twitterRequestWithIdentifier:requestIdentifier];
    
    if(req != nil){
        [req setSuccessfulValue:YES];
		if([req requestType] == credentialsTest) {
			[[NSNotificationCenter defaultCenter] postNotificationName:TwitterCredentialsValid object:self];
		}
    }
}

- (void)requestFailed:(NSString *)requestIdentifier withError:(NSError *)error{
    LLTwitterRequest * req = [self twitterRequestWithIdentifier:requestIdentifier];
    
    if(req){
        [req setSuccessfulValue:NO];
        if([req requestType] == credentialsTest){
			[[NSNotificationCenter defaultCenter] postNotificationName:TwitterCredentialsInvalid object:self];
		}
		
		[self saveContextChanges];
    }
}

- (void)statusesReceived:(NSArray *)statuses forRequest:(NSString *)identifier{
    LLTwitterRequest * req = [self twitterRequestWithIdentifier:identifier];
    [statuses retain];
    [self managedTwitterUsersFromDictionaries:statuses];
    
    if(req){
		
    }
    [statuses release];
}

- (void)directMessagesReceived:(NSArray *)messages forRequest:(NSString *)identifier{
    LLTwitterRequest * req = [self twitterRequestWithIdentifier:identifier];
    
    if(req){
        
    }
}

- (void)userInfoReceived:(NSArray *)userInfo forRequest:(NSString *)identifier{
    LLTwitterRequest * req = [self twitterRequestWithIdentifier:identifier];
    
    if(req){
        
    }
}

- (void)miscInfoReceived:(NSArray *)miscInfo forRequest:(NSString *)identifier{
    LLTwitterRequest * req = [self twitterRequestWithIdentifier:identifier];
    
    if(req){
        
    }
}

- (void)imageReceived:(UIImage *)image forRequest:(NSString *)identifier{
    LLTwitterRequest * req = [self twitterRequestWithIdentifier:identifier];
    
    if(req){
        
    }
}

#pragma mark Method for performing requests using request objects

-(void)performRequest:(LLTwitterRequest *)req{
    if([req requestType] == targetInfo){
		NSString * forName = [[req targetUser] username];
		NSString * identifier = [twitterEngine getUserInformationFor:forName];
		[req setRequestIdentifier:identifier];
	}else if([req requestType] == targetTweets){
		NSString * forName = [[req targetUser] username];
		NSDate * date = [req sinceDate];
		NSString * identifier = [twitterEngine getUserTimelineFor:forName since:date count:50];
		[req setRequestIdentifier:identifier];
	}else if([req requestType] == usersFollowing){
		NSString * identifier = [twitterEngine getFollowersIncludingCurrentStatus:NO]; 
		[req setRequestIdentifier:identifier];
	}else if([req requestType] == followingUsers){
		[req setRequestIdentifier:[twitterEngine getFollowersIncludingCurrentStatus:NO]];
	}else if([req requestType] == credentialsTest){
		[req setRequestIdentifier:[twitterEngine checkUserCredentials]];
	}
	[self saveContextChanges];
}

#pragma mark Convenience Methods For Accessing 

#pragma mark - Username Lists
-(void)getFollowingUsersFor:(LLTwitterUser *)twitterUser autoAddResults:(BOOL)addResults{
    LLTwitterRequest * request = (LLTwitterRequest *)[self newObjectForName:@"LLTwitterRequest"];
    [request setRequestType:followingUsers];
	[request setTargetUser:twitterUser];
    [self performRequest:request];
}

-(void)getUsersFollowingAndAutoAddResuults:(BOOL)addResults{
    LLTwitterRequest * request = (LLTwitterRequest *)[self newObjectForName:@"LLTwitterRequest"];
    [request setRequestType:usersFollowing];
	//[request setTargetUser:twitterUser];
    [self performRequest:request];
}

#pragma mark - Tweets

-(void)getLatestUpdatesForUser:(NSString *)username{
    LLTwitterRequest * request = (LLTwitterRequest *)[self newObjectForName:@"LLTwitterRequest"];
    [request setRequestType:targetTweets];
    [request setTargetUser:[self twitterUserForUsername:username createIfAbsent:YES]];
    [self performRequest:request];
}

#pragma mark - User Credentials

-(void)checkCurrentCredentials{
    LLTwitterRequest * request = (LLTwitterRequest *)[[self coreDataManager] newObjectForName:@"LLTwitterRequest"];
    [request setRequestType:credentialsTest];
    [self performRequest:request];
    [[NSNotificationCenter defaultCenter] postNotificationName:TwitterCredentialsChecking object:self];
}

-(void)setCredentialsWithUsername:(NSString *)username withPassword:(NSString *)password{
    [twitterEngine setUsername:username password:password];
	[[LLUserDefaults sharedLLUserDefaults] setStoreCredentialsWithUsername:username withPassword:password];
    [self checkCurrentCredentials];
}

#pragma mark Methods unique to jogcastr

-(NSArray *)startRaceStatusMessagesToBroadcastToUsers:(NSArray *) users{
    uint userNamesTotalChars = 0;
    for(NSString * userName in users){
        userNamesTotalChars += [userName length] + 2;
    }
        //uint maxUsernameCharsPerMessage = MAX_MESSAGE_LENGTH - [NotifyUsersStartRace length];
    NSMutableArray * messages = [NSMutableArray arrayWithCapacity:10];
    NSMutableString * currentString = [NSMutableString stringWithFormat:@"%@ %@",JogcastrHashtag,NotifyUsersStartRace];
    for(NSString * userName in users){
        if(([currentString length] + [userName length] + 2) < MAX_MESSAGE_LENGTH){
            [currentString appendFormat:@"@%",userName];
        }else {
            [messages addObject:currentString];
            currentString = [NSMutableString stringWithFormat:@"%@ %@",JogcastrHashtag,NotifyUsersStartRace];
        }
    }
    return messages;
}

-(void)sendLocationUpdate:(LLSendMessageModel *)message{
    [twitterEngine setLocation:[message locationLatLong]];
    [twitterEngine sendUpdate:[NSString stringWithFormat:JogcastrHashtag]];
}

-(void)stopViewingRunnerUpdates{
    if(updateCheckTimer != nil){
        if([updateCheckTimer isValid]) 
            [updateCheckTimer invalidate];
    }
}

-(void)startViewingRunnerUpdates{
        //[twitterEngine getUserInformationFor:viewingRunner];
        //updateCheckTimer = [NSTimer scheduledTimerWithTimeInterval:FOLLOWING_UPDATE_INTERVAL target:self selector:@selector(checkForUpdates) userInfo:nil repeats:YES];
}

- (void)checkForUpdates{
    NSLog(@"Timer Fired");
}

@end

#pragma mark Private Method Implementation

@implementation LLTwitterController (PrivateMethods)

-(void)setCredentialsFromStore{
	NSDictionary * credentials = [[LLUserDefaults sharedLLUserDefaults] storeCredentials];
	[self setCredentialsWithUsername:[credentials objectForKey:UsernameKey] withPassword:[credentials objectForKey:PasswordKey]];
}

@end