//
//  UIButton+Glossy.m
//  LLJogTest
//
//  Created by Lawrence Lomax on 14/01/2010.
//  Copyright 2010 Lawrence Lomax. All rights reserved.
//

#import "LLGlossyUIButton+Glossy.h"

@implementation LLGlossyUIButton(Glossy)

+ (void)setPathToRoundedRect:(CGRect)rect forInset:(NSUInteger)inset inContext:(CGContextRef)context
{
        // Experimentally determined
	static NSUInteger cornerRadius = 10;
    
        // Unpack size for compactness, find minimum dimension
	CGFloat w = rect.size.width;
	CGFloat h = rect.size.height;
	CGFloat m = w<h?w:h;
    
        // Bounds
	CGFloat b = rect.origin.y;
	CGFloat t = b + h;
	CGFloat l = rect.origin.x;
	CGFloat r = l + w;
	CGFloat d = (inset<cornerRadius)?(cornerRadius-inset):0;
    
        // Special case: Degenerate rectangles abort this method
	if (m <= 0) return;
    
        // Limit radius to 1/2 of the rectangle's shortest axis
	d = (d>0.5*m)?(0.5*m):d;
    
        // Define a CW path in the CG co-ordinate system (origin at LL)
	CGContextBeginPath(context);
	CGContextMoveToPoint(context, (l+r)/2, t);		// Begin at TDC
	CGContextAddArcToPoint(context, r, t, r, b, d);	// UR corner
	CGContextAddArcToPoint(context, r, b, l, b, d);	// LR corner
	CGContextAddArcToPoint(context, l, b, l, t, d);	// LL corner
	CGContextAddArcToPoint(context, l, t, r, t, d);	// UL corner
	CGContextClosePath(context);					// End at TDC
}

- (UIButton*)buttonWithWidth:(NSUInteger)width height:(NSUInteger)height
{
        // Create button (in UL corner of client area)
	UIButton* button = [UIButton buttonWithType:UIButtonTypeRoundedRect];
	button.frame = CGRectMake(0, 0, width, height);
    
        // Add to TL view, and return
	[self addSubview:button];
	return button;
}
- (CGContextRef)currentImageContextWithWidth:(NSUInteger)width height:(NSUInteger)height
{
        // Create and get a pointer to context
	UIGraphicsBeginImageContext(CGSizeMake(width, height));
	CGContextRef context = UIGraphicsGetCurrentContext();
    
        // Convert co-ordinate system to Cocoa's (origin in UL, not LL)
	CGContextTranslateCTM(context, 0, height);
	CGContextConcatCTM(context, CGAffineTransformMakeScale(1, -1));
    
        // Set fill and stroke colors
	CGContextSetFillColorWithColor(context, [[UIColor colorWithRed:0.65 green:0.85 blue:0.85 alpha:1] CGColor]);
	CGContextSetStrokeColorWithColor(context, [[UIColor colorWithRed:159.0/255 green:159.0/255 blue:159.0/255 alpha:1] CGColor]);
    
        // Return context
	return context;
}
- (UIImage*)imageFromCurrentContext
{
	UIImage* image = UIGraphicsGetImageFromCurrentImageContext();
	UIGraphicsEndImageContext();
	return image;
}
- (UIImageView*)imageViewFromCurrentContext
{
	UIImageView* imageView = [[[UIImageView alloc] initWithImage:[self imageFromCurrentContext]] autorelease];
	[self addSubview:imageView];
	return imageView;
}

    // Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView
{
	UIButton*	button;
	CGContextRef	context;
	UIImage*	image;
	UIImageView*	imageView;
    
        // TL view
	self = [[UIView alloc] initWithFrame:CGRectZero];
	self.backgroundColor = [UIColor whiteColor];
    
        // Create a basic rounded rect button
	[self buttonWithWidth:240 height:40].center = CGPointMake(160, 40);
    
        // Create drawing context for a rounded rectangle's border
	context = [self currentImageContextWithWidth:240 height:40];
        // Add stroking path
        // * Runs through pixel centers
        // * Dimensions reduced by 1 to allow for 1-pixel stroke width
	[LLGlossyUIButton setPathToRoundedRect:CGRectMake(0.5, 0.5, 240-1, 40-1) forInset:0 inContext:context];
        // Stroke path
	CGContextStrokePath(context);
        // Create and position image
	[self imageViewFromCurrentContext].center = CGPointMake(160, 100);
    
        // Create drawing context for a rectangular image
	context = [self currentImageContextWithWidth:240 height:40];
        // Fill image
	CGContextFillRect(context, CGRectMake(0, 0, 240, 40));
        // Create and position image
	[self imageViewFromCurrentContext].center = CGPointMake(160, 160);
    
        // Create drawing context for a rounded image
	context = [self currentImageContextWithWidth:240 height:40];
        // Add clipping path
        // * Runs around the perimeter of the included area
        // * Dimensions are *not* reduced, as path is a zero-thickness boundary
	[LLGlossyUIButton setPathToRoundedRect:CGRectMake(0, 0, 240, 40) forInset:0 inContext:context];
	CGContextClip(context);
        // Fill image
	CGContextFillRect(context, CGRectMake(0, 0, 240, 40));
        // Create and position image
	[self imageViewFromCurrentContext].center = CGPointMake(160, 220);
    
        // Create drawing context for an inset rounded image
        // Dimensions are reduced by 2, to allow for a 1-pixel surrounding border
	context = [self currentImageContextWithWidth:240-2 height:40-2];
        // Add clipping path
        // * Runs around the perimeter of the included area
        // * Dimensions are *not* (further) reduced, as path is a zero-thickness boundary
        // * A path is created "forInset" 1:
        //   . When one rounded corner is placed inside another, the interior
        //     corner must have its radius reduced for a proper appearance
	[LLGlossyUIButton setPathToRoundedRect:CGRectMake(0, 0, 240-2, 40-2) forInset:1 inContext:context];
	CGContextClip(context);
        // Fill image
	CGContextFillRect(context, CGRectMake(0, 0, 240-2, 40-2));
        // Get image
	image = [self imageFromCurrentContext];
    
        // Draw the inset image, both alone, and inside a rounded rect button's border
	imageView = [[[UIImageView alloc] initWithImage:image] autorelease];
	[self addSubview:imageView];
	imageView.center = CGPointMake(160, 280);
    
	button = [self buttonWithWidth:240 height:40];
	[button setImage:image forState:UIControlStateNormal];
	button.center = CGPointMake(160, 340);
}

@end
