//
//  LLUserDefaults.m
//  LLJogTest
//
//  Created by Lawrence Lomax on 07/03/2010.
//  Copyright 2010 Lawrence Lomax. All rights reserved.
//

#import "LLUserDefaults.h"
#import "LLCoreDataManager.h"

NSString * const UsernameKey = @"username";
NSString * const PasswordKey = @"password";
NSString * const JourneyIDKey = @"active_journey_id_key";
NSString * const AutoRestartKey = @"auto_restart_journey_key";
NSString * const CurrentUserKey = @"current_user_key";

@implementation LLUserDefaults

@synthesize userDefaults;

SYNTHESIZE_SINGLETON_FOR_CLASS(LLUserDefaults)

-(id)init{
	if(self = [super init]){
		userDefaults = [NSUserDefaults standardUserDefaults];
	}
	return self;
}

-(NSDictionary *)storeCredentials{
    NSString *username = [userDefaults stringForKey:UsernameKey];
    NSString *password = [userDefaults stringForKey:PasswordKey];
    
    if(!username || !password){
        username = @"";
        password = @"";
    }
	NSArray * objectArray = [NSArray arrayWithObjects:username,password,nil];
	NSArray * keyArray = [NSArray arrayWithObjects:UsernameKey,PasswordKey,nil];
	
	return [NSDictionary dictionaryWithObjects:objectArray forKeys:keyArray];
}

-(void)setStoreCredentialsWithUsername:(NSString *)username withPassword:(NSString *)password{
    NSUserDefaults * prefs = [NSUserDefaults standardUserDefaults];
    
    NSString *storeUsername = [userDefaults stringForKey:UsernameKey];
    NSString *storePassword = [userDefaults stringForKey:PasswordKey];
    
    if(!storeUsername){
        if([username isEqualToString:storeUsername]){
			[prefs setObject:username forKey:UsernameKey];
		}
	}
	if(!storePassword){
		if([username isEqualToString:storePassword]){
			[prefs setObject:password forKey:PasswordKey];
        }
	}
	[prefs synchronize];
}

#pragma mark Auto Restart
-(BOOL)autoRestartJourney{
	return [userDefaults boolForKey:AutoRestartKey];
}

-(void)setAutoRestartJourney:(BOOL)autoRestart{
	[userDefaults setBool:autoRestart forKey:AutoRestartKey];
}

#pragma mark Current User from Store
-(LLTwitterUserID *)currentTwitterUserID{
	return (LLTwitterUserID *)[self managedObjectIDWithKey:CurrentUserKey];
}
-(LLTwitterUser *)currentTwitterUser{
	return (LLTwitterUser *)[self managedObject:CurrentUserKey];
}
-(void)setCurrentTwitterUserID:(LLTwitterUserID *)userID{
	[self setManagedObjectID:userID withKey:CurrentUserKey];
}
-(void)setCurrentTwitterUser:(LLTwitterUser *)user{
	[self setManagedObject:user withKey:CurrentUserKey];
}

#pragma mark Active Journey 
-(LLJourneyID *)activeJourneyID{
	return (LLJourneyID *)[self managedObjectIDWithKey:JourneyIDKey];
}

-(LLJourney *)activeJourney{
	return (LLJourney *)[self managedObject:JourneyIDKey];
}

-(void)setActiveJourneyID:(LLJourneyID *)journeyID{
	[self setManagedObjectID:journeyID withKey:JourneyIDKey]; 
}

-(void)setActiveJourney:(LLJourney *)journey{
	[self setManagedObject:journey withKey:JourneyIDKey];
}

-(NSManagedObjectID *)managedObjectIDWithKey:(NSString *)key{
	NSString * urlString = [userDefaults objectForKey:JourneyIDKey];
	if(urlString){
		NSURL * moIDURL = [NSURL URLWithString:urlString];
		NSManagedObjectID * objectID = [[[LLCoreDataManager sharedLLCoreDataManager] persistentStoreCoordinator] managedObjectIDForURIRepresentation:moIDURL];
		return objectID;
	}else {
		return nil;
	}
}

-(NSManagedObject *)managedObject:(NSString *)key{
	NSManagedObjectID * objectID = [self managedObjectIDWithKey:key];
	if(objectID)
		return [[[LLCoreDataManager sharedLLCoreDataManager] managedObjectContext] objectWithID:objectID];
	else
		return nil;
}
									
-(void)setManagedObjectID:(NSManagedObjectID *)objectID withKey:(NSString *)key{
	NSURL * objectURL = [objectID URIRepresentation];
	NSString * urlString = [objectURL absoluteString];
	[userDefaults setObject:urlString forKey:key];
}

-(void)setManagedObject:(NSManagedObject *)object withKey:(NSString *)key{
	[self setManagedObjectID:[object objectID] withKey:key];
}

@end
