//
//  LLOptionsTableViewController.m
//  LLJogTest
//
//  Created by Lawrence Lomax on 13/03/2010.
//  Copyright 2010 Apple Inc. All rights reserved.
//

#import "LLOptionsTableViewController.h"
#import "NSObject+CollectionsUtilities.h"
#import "LLUserDefaults.h"

NSString * const Options_Section_Title = @"Option";
NSString * const Help_Section_Title = @"Help"; 

NSString * const Twitter_Account_Option = @"Twitter Account";
NSString * const Auto_Restart_Active_Journey_Option = @"Auto Restart Active Journey";

@implementation LLOptionsTableViewController


#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
	
	self.navigationItem.title = @"Help & Options";	//Title Name
	//self.navigationItem. = @"Options";
	
	
	NSArray * optionsSectionElements = [NSArray arrayWithObjects:Twitter_Account_Option,Auto_Restart_Active_Journey_Option,nil];
	NSArray * helpSectionElements = [NSArray array];
	
	NSDictionary * optionsSection = [NSDictionary dictionaryWithObject:optionsSectionElements forKey:Options_Section_Title];
	NSDictionary * helpSection = [NSDictionary dictionaryWithObject:helpSectionElements forKey:Help_Section_Title];
	
	sectionsArray = [NSArray arrayWithObjects:optionsSection,helpSection,nil];
	[sectionsArray retain];

	// Uncomment the following line to preserve selection between presentations.
    //self.clearsSelectionOnViewWillAppear = NO;
 
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
}

/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}
*/
/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/


#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return [sectionsArray count];
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
    return [self numberOfRowsInSectionsInArrayMap:sectionsArray withSectionNumber:section];   
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
	NSString * cellName = [self rowNameWithArrayMap:sectionsArray withSectionNumber:indexPath.section withRowNumber:indexPath.row];
	cell.textLabel.text = cellName;
	
	if(cellName == Auto_Restart_Active_Journey_Option){
		if([[LLUserDefaults sharedLLUserDefaults] autoRestartJourney])
			cell.accessoryType = UITableViewCellAccessoryCheckmark;
		else
			cell.accessoryType = UITableViewCellAccessoryNone;
	}
	
	if(cellName == Twitter_Account_Option){
		cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
	}
	
    // Configure the cell...
    
    return cell;
}

//Sections heaers for Table View
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section{
	NSString * sectionName = [self sectionNameWithArrayMap:sectionsArray withSectionNumber:section];
	return sectionName;
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:YES];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	NSString * selectedSectionName = [self sectionNameWithArrayMap:sectionsArray withSectionNumber:indexPath.section];
	NSString * selectedRowName = [self rowNameWithArrayMap:sectionsArray withSectionNumber:indexPath.section withRowNumber:indexPath.row];
	
	if(selectedSectionName == Options_Section_Title){
		if(selectedRowName == Twitter_Account_Option){
			if(!accountOptionsViewController){
				accountOptionsViewController = [[LLTwitterAccountViewController alloc] initWithNibName:@"LLTwitterAccountView" bundle:[NSBundle mainBundle]];
			}
			
			[self.navigationController pushViewController:accountOptionsViewController animated:YES];
		}else if(selectedRowName == Auto_Restart_Active_Journey_Option){
			LLUserDefaults * userDefaults = [LLUserDefaults sharedLLUserDefaults]; 
			[userDefaults setAutoRestartJourney:![userDefaults autoRestartJourney]];
			[userDefaults.userDefaults synchronize];
			[tableView reloadData];
		}	
	}
	
    // Navigation logic may go here. Create and push another view controller.
	/*
	 <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
	 [self.navigationController pushViewController:detailViewController animated:YES];
	 [detailViewController release];
	 */
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
    [sectionsArray release];
	[super dealloc];
}


@end

