//
//  LLRootTableViewController.m
//  LLJogTest
//
//  Created by Lawrence Lomax on 13/03/2010.
//  Copyright 2010 Apple Inc. All rights reserved.
//

#import "LLRootTableViewController.h"
#import "NSObject+CollectionsUtilities.h"
#import "LLUserDefaults.h"

NSString * const My_Journeys_Section_Name = @"My Journeys";
NSString * const Friends_Journeys_Section_Name = @"Friends";
NSString * const Help_And_Options_Section_Name = @"Help & Options";

NSString * const Start_Journey_Item_Name = @"Start Journey";
NSString * const Past_Journeys_Item_Name = @"Past Journeys";

NSString * const Friends_Item_Name = @"Friends";

NSString * const Help_And_Options_Item_Name = @"Help & Options";


@implementation LLRootTableViewController


#pragma mark -
#pragma mark View lifecycle


- (void)viewDidLoad {
    [super viewDidLoad];
	
	NSDictionary * myJourneysSection = [NSDictionary dictionaryWithObject:[NSArray arrayWithObjects:Start_Journey_Item_Name, Past_Journeys_Item_Name,nil] forKey:My_Journeys_Section_Name];
	NSDictionary * friendsSection = [NSDictionary dictionaryWithObject:[NSArray arrayWithObjects:Friends_Item_Name,nil] forKey:Friends_Journeys_Section_Name];
	NSDictionary * helpAndOptionsSection = [NSDictionary dictionaryWithObject:[NSArray arrayWithObjects:Help_And_Options_Item_Name,nil] forKey:Help_And_Options_Section_Name];
	
	sectionsArray = [NSArray arrayWithObjects:myJourneysSection,friendsSection,helpAndOptionsSection,nil];
	[sectionsArray retain];
	
	LLUserDefaults * userDefaults = [LLUserDefaults sharedLLUserDefaults];
	if([userDefaults autoRestartJourney]){
		if([userDefaults activeJourneyID]){
			if(!activeJourneyViewController){
				activeJourneyViewController = [[LLActiveJourneyViewController alloc] initWithNibName:@"LLActiveJourneyView" bundle:[NSBundle mainBundle]];
			}
			[self.navigationController pushViewController:activeJourneyViewController animated:YES];
		}	
	}
	
    // Uncomment the following line to preserve selection between presentations.
    //self.clearsSelectionOnViewWillAppear = NO;
 
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
}


/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}
*/
/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/


#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return [sectionsArray count];
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
    return [self numberOfRowsInSectionsInArrayMap:sectionsArray withSectionNumber:section];
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
	
	NSString * itemName = [self rowNameWithArrayMap:sectionsArray withSectionNumber:indexPath.section withRowNumber:indexPath.row];
    cell.textLabel.text = itemName;
	
    // Configure the cell...
    
    return cell;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section{
	NSString * sectionName = [self sectionNameWithArrayMap:sectionsArray withSectionNumber:section];
	return sectionName;
}

/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:YES];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    NSString * selectedSectionName = [self sectionNameWithArrayMap:sectionsArray withSectionNumber:indexPath.section];
	NSString * selectedRowName = [self rowNameWithArrayMap:sectionsArray withSectionNumber:indexPath.section withRowNumber:indexPath.row];
	
	if(selectedSectionName == My_Journeys_Section_Name){
		if(selectedRowName == Start_Journey_Item_Name){
			if(!activeJourneyViewController){
				activeJourneyViewController = [[LLActiveJourneyViewController alloc] initWithNibName:@"LLActiveJourneyView" bundle:[NSBundle mainBundle]];
			}
			[self.navigationController pushViewController:activeJourneyViewController animated:YES];
		}else if(selectedRowName == Past_Journeys_Item_Name){
		
		}
	}else if(selectedSectionName == Friends_Journeys_Section_Name){
		if(selectedRowName == Friends_Item_Name){
		
		}
	}else if(selectedSectionName == Help_And_Options_Item_Name){
		if(selectedRowName == Help_And_Options_Item_Name){
			if(!optionsTableViewController){
				optionsTableViewController = [[LLOptionsTableViewController alloc] initWithNibName:@"LLOptionsTableView" bundle:[NSBundle mainBundle]];
			}
			[self.navigationController pushViewController:optionsTableViewController animated:YES];
			
		}
	}
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
	[sectionsArray release];
    [super dealloc];
}


@end

